 /*
 *	Rush PSX R3000A Emulation Library header - v1.0 Alpha
 *
 *	http://psxdev.narod.ru/docs/rush.htm
 *	for suggestions, comments and etc - kvzorganic@mail.ru
 *	(C) Organic /2002/
 */

#ifndef	__RUSH_H__
#define	__RUSH_H__	1

/* check platform */
#ifndef WIN32
#error Please, use Win32 C compiler!
#endif	// __WIN32__

typedef int	RUSH_RESULT;

/* CPU types */
enum	RUSH_CPU_TYPE
{
	RUSH_CPU_INTERPRETER,
	RUSH_CPU_DYNAREC,
	RUSH_CPU_DEBUGGER,
	RUSH_CPU_STATIREC,
	RUSH_CPU_UNKNOWN = -1	/* use for autodetect */
};

/* Rush flags */
#define RUSH_HLE_ENABLE		1
#define RUSH_HLE_DISABLE	0
#define RUSH_RAND_ENABLE	2
#define RUSH_RAND_DISABLE	0

/* Rush return codes */
#define RUSH_OKAY	0
#define	RUSH_ERR	-1

/* defines for Lock/UnLock */
enum	RUSH_LOCK_AREA
{
	RUSH_LOCK_RAM,
	RUSH_LOCK_BIOS,
	RUSH_LOCK_HARDWARE,
	RUSH_LOCK_PORT,
	RUSH_LOCK_PAD
};


/* Rush Get selectors */
enum	RUSH_GET_SELECTOR
{
	RUSH_GET_NAME,
	RUSH_GET_VERSION,
	RUSH_GET_CLOCK_COUNTER,
	RUSH_GET_INSTRUCTION,
	RUSH_GET_CPU_REGISTERS,
	RUSH_GET_COP0_REGISTERS,
	RUSH_GET_HI_REGISTER,
	RUSH_GET_LO_REGISTER,
	RUSH_GET_PROGRAM_COUNTER
};

/* exception codes */
enum	RUSH_EXCEPTION_TYPE
{
	RUSH_XCPT_RESET,
	RUSH_XCPT_INTERRUPT,
	RUSH_XCPT_BADOPCODE,
	RUSH_XCPT_SYSCALL,
	RUSH_XCPT_BREAK,
	RUSH_XCPT_OVERFLOW,

	RUSH_XCPT_UNKNOWN = -1
};


typedef struct
{
	/* cpu */
	RUSH_RESULT (*Init)(void);
	void (*Shutdown)(void);
	void (*Execute)(void);
	void (*Exception)(enum RUSH_EXCEPTION_TYPE);
	void (*Interrupt)(void);
	
	/* memory */
	unsigned long *(*Lock)(enum RUSH_LOCK_AREA area);
	void (*UnLock)(void);
	int (*SetHook8)(unsigned long hwaddr, unsigned char (*ReadData)(), void (*WriteData)(unsigned char data));
	int (*KillHook8)(unsigned long hwaddr);
	int (*SetHook16)(unsigned long hwaddr, unsigned short (*ReadData)(), void (*WriteData)(unsigned short data));
	int (*KillHook16)(unsigned long hwaddr);
	int (*SetHook32)(unsigned long hwaddr, unsigned long (*ReadData)(), void (*WriteData)(unsigned long data));
	int (*KillHook32)(unsigned long hwaddr);

	/* get some stuff */
	void *(*Get)(enum RUSH_GET_SELECTOR);

	/* debugging */
	char *(*Debug)(unsigned long);
	void (*SetPC)(unsigned long);
	unsigned long (*Assembly)(char *);
} R3000A;


#endif	/* __RUSH_H__ */